unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeeOpenGL, TeeComma, ExtCtrls, TeeProcs, TeEngine, Chart;

type
  TForm1 = class(TForm)
    CellDisplay: TChart;
    TeeCommanderCellDisplay: TTeeCommander;
    TeeOpenGL: TTeeOpenGL;
    procedure FormCreate(Sender: TObject);
    procedure CellDisplayAfterDraw(Sender: TObject);
  private
    { Private declarations }
    FPieChartRects: array of TRect;
    FXPositions: array[0..95] of double;
    FYPositions: array[0..95] of double;
    procedure DrawCellDisplay;
    procedure PieSeriesBeforeDraw(Sender: TObject);
    procedure CreateCellDisplayPieSeries;
    procedure CreateXYGrid(HorizontalDistance,VerticalDistance: Double);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses Series, Math;

{$R *.dfm}

procedure TForm1.CreateXYGrid(HorizontalDistance,VerticalDistance: Double);
var
  i: Integer;
  HorizontalCount,VerticalCount,NoOfItems: Integer;
begin
  for i := 0 to 95 do
  begin
    FXPositions[i] := 0;
    FYPositions[i] := 0;
  end;
  HorizontalCount := 0;
  VerticalCount := 0;
  NoOfItems := 11;
  for i := 0 to 95 do
  begin
    FXPositions[i] := HorizontalCount * HorizontalDistance;
    FYPositions[i] := VerticalCount * VerticalDistance;
    if HorizontalCount = NoOfItems then
    begin
      HorizontalCount := 0;
      Inc(VerticalCount);
    end else
      Inc(HorizontalCount);
  end;
end;	

procedure TForm1.CellDisplayAfterDraw(Sender: TObject);
begin
  DrawCellDisplay;
end;

procedure TForm1.CreateCellDisplayPieSeries;
var
  tmpPieSeries: TPieSeries;
  i: Integer;
begin
  CellDisplay.AutoRepaint := False;
  try
    CellDisplay.SeriesList.Clear;
    for i := 0 to 95 do
    begin
      tmpPieSeries := TPieSeries.Create(CellDisplay);
      tmpPieSeries.BeginUpdate;
      try
        tmpPieSeries.ParentChart := CellDisplay;
        tmpPieSeries.Title := IntToStr(i + 1);
        tmpPieSeries.Tag := i;
        tmpPieSeries.Marks.Visible := False;
        tmpPieSeries.CustomXRadius := 35;
        tmpPieSeries.CustomYRadius := 35;
        tmpPieSeries.Depth := 10;
        tmpPieSeries.BeforeDrawValues := PieSeriesBeforeDraw;
        tmpPieSeries.Add(i, IntToStr(i + 1), clRed);
        tmpPieSeries.Visible := True;
      finally
        tmpPieSeries.EndUpdate;
        tmpPieSeries.Active := False;
      end;
    end;
  finally
    CellDisplay.AutoRepaint := True;
  end;
end;

procedure TForm1.PieSeriesBeforeDraw(Sender: TObject);
begin
  if Sender is TPieSeries then
    if TPieSeries(Sender).Tag < Length(FPieChartRects) then
      CellDisplay.ChartRect := FPieChartRects[TPieSeries(Sender).Tag];
end;

procedure TForm1.DrawCellDisplay;
var
  i: Integer;
  MinXPos,MaxXPos,MinYPos,MaxYPos: Integer;
  MinX,MinY,MaxX,MaxY: double;
  XPos,YPos: Integer;
  SampleDiameter: Integer;
  LeftX,TopY: Integer;
  tmpX,tmpY,tmpZ: Integer;
  CellDisplaySizeFactor: Double;
  LeftCube,RightCube,TopCube,BottomCube,Z0Cube,Z1Cube: Integer;
begin
  SetLength(FPieChartRects, 96);
  CellDisplaySizeFactor := 10;
  SampleDiameter := 46;
  MinX := 100000;
  MaxX := -100000;
  MinY := 100000;
  MaxY := -100000;
  for i := 0 to 95 do
  begin
    MinX := Min(MinX, FXPositions[i]);
    MaxX := Max(MaxX, FXPositions[i]);
    MinY := Min(MinY, FYPositions[i]);
    MaxY := Max(MaxY, FYPositions[i]);
  end;
  MinXPos := Round(CellDisplaySizeFactor * MinX);
  MaxXPos := Round(CellDisplaySizeFactor * MaxX);
  MinYPos := Round(CellDisplaySizeFactor * MinY);
  MaxYPos := Round(CellDisplaySizeFactor * MaxY);
  with CellDisplay, Canvas do
  begin
    tmpY := ChartYCenter + Round(RotationCenter.Y);
    tmpX := ChartXCenter + Round(RotationCenter.X);
    tmpZ := (Width3D div 2) + Round(RotationCenter.Z);
    LeftX := ChartXCenter - (Round((MaxXPos - MinXPos) / 2) + MinXPos);
    TopY := ChartYCenter - (Round((MaxYPos - MinYPos) / 2) + MinYPos);
    LeftCube := LeftX + MinXPos - SampleDiameter;
    RightCube := LeftX + MaxXPos + SampleDiameter;
    TopCube := TopY + MinYPos - SampleDiameter;
    BottomCube := TopY + MaxYPos + SampleDiameter;
    Z0Cube := tmpZ + 42;
    Z1Cube := tmpZ + 48;
    Cube(LeftCube, RightCube, TopCube, BottomCube, Z0Cube, Z1Cube);
    for i := 0 to 95 do
    begin
      XPos := Round(FXPositions[i] * CellDisplaySizeFactor);
      YPos := Round(FYPositions[i] * CellDisplaySizeFactor);
      FPieChartRects[i] := Rect(LeftX + XPos - Round(CellDisplay.Width / 10), TopY + YPos - Round(CellDisplay.Width / 10),
                                LeftX + XPos + Round(CellDisplay.Width / 10), TopY + YPos + Round(CellDisplay.Width / 10));
    end;
    Pen.Color := clBlue;
    Pen.Width := 2;
    Pen.Style := psDot;
    HorizLine3D(ChartRect.Left, ChartRect.Right, tmpY, tmpZ);
    VertLine3D(tmpX, ChartRect.Top, ChartRect.Bottom, tmpZ);
    ZLine3D(tmpX, tmpY, 0, Width3D);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  CreateXYGrid(15, 15);
  CreateCellDisplayPieSeries;
  for i := 0 to CellDisplay.SeriesCount - 1 do
    CellDisplay.Series[i].Visible := True;
  DrawCellDisplay;
  CellDisplay.ApplyZOrder := False;
  CellDisplay.View3DOptions.Zoom := 20;
  CellDisplay.View3DOptions.Rotation := 19;
  CellDisplay.View3DOptions.Elevation := 336;
end;

end.
